<?php namespace EscapeWork\Manager\Medias\Models;

use EscapeWork\Manager\Medias\Repositories\MediaRepository;
use EscapeWork\Manager\Medias\Models\MediaModelSize;
use EscapeWork\Manager\Medias\Upload;
use EscapeWork\LaravelHelpers\Eloquent;

class Media extends Eloquent implements MediaRepository
{

    /**
     * Table
     */
    protected $table = 'medias';

    /**
     * Regras para validação
     */
    public static $validationRules = array(
        'media_model' => array('required'),
        'model_id'    => array('required'),
        'file'        => array('required'),
        'mime'        => array('required'),
        'size'        => array('required'),
        'order'       => array('integer'),
    );

    /**
     * Mensagens para validação
     */
    public static $validationMessages = array(
        'model.required' => 'O nome da tabela é obrigatório'
    );

    /**
     * Fields
     */
    public $fillable = array(
        'id',
        'media_model',
        'model_id',
        'file',
        'mime',
        'size',
        'legend',
        'credits',
        'order',
        'active'
    );

    /**
     * Pasta de localização
     */
    public static $url = '/files/medias/';
    public static $dir = '/files/medias/';

    public static function dir()
    {
        return public_path() . static::$dir;
    }

    /**
     * Scope for active medias only
     */
    public function scopeActive($query)
    {
        return $query->where('active', '=', true);
    }

    /**
     * Scope for default get Medias from
     */
    public function scopeGetActualModel($query)
    {
        return $query->where('media_model', '=', $this->media_model)
                    ->where('model_id', '=', $this->model_id);
    }

    /**
     * Setando o media_model_id atravês do nome do Model
     */
    public function setModel($model)
    {
        $this->media_model = $model;
    }

    /**
     * Inserindo uma mídia
     */
    public function insert($picture, $attrs = array())
    {
        $upload = Upload::create(
            static::dir(),
            $picture,
            $this->media_model
        );

        if (! ($upload instanceof Upload)) {
            return;
        }

        $media              = new Media($attrs);
        $media->media_model = $this->media_model;
        $media->model_id    = $this->model_id;
        $media->file        = $upload->getName();
        $media->mime        = $upload->getMime();
        $media->size        = $upload->getSize();
        $media->order       = $this->getLastOrder();
        return $media->save();
    }

    /**
     * Polymorphical relation
     */
    public function medias()
    {
        return $this->morphTo();
    }

    /**
     * Returning the last order for the actual model
     */
    protected function getLastOrder()
    {
        $media = $this->getActualModel()
                      ->orderBy('order', 'DESC')
                      ->first();

        if ($media) {
            return $media->order + 1;
        }

        return 1;
    }

    /**
     * Retornando as mídias do determinado model e model relacionado
     */
    public function getMedias()
    {
        return $this->getActualModel()
                ->orderBy('order')
                ->orderBy('created_at', 'DESC')
                ->get();
    }

    /**
     * Retornando a principal foto
     */
    public function getMedia()
    {
        return $this->getActualModel()
                ->orderBy('order')
                ->orderBy('created_at', 'DESC')
                ->take(1)
                ->first();
    }

    /**
     * Excluindo a mídia
     */
    public function delete()
    {
        $this->deleteFiles();

        return parent::delete();
    }

    /**
     * Excluindo todas as mídias de um model
     *
     * @param  string media_model
     * @param  int    model_id
     */
    public function deleteAll()
    {
        $medias = $this->getMedias();

        foreach ($medias as $media) {
            $media->delete();
        }
    }

    /**
     * Excluindo os arquivos
     */
    public function deleteFiles()
    {
        $sizes = MediaModelSize::getSizes($this->media_model);

        foreach ($sizes as $size) {
            $file = static::dir() . $size['prefix'] . $this->file;
            if (is_file($file)) {
                unlink($file);
            }
        }

        $file = static::dir() . $this->file;
        if (is_file($file)) {
            unlink($file);
        }
    }
}
